@@ -38,11 +38,10 @@
 #include <libxfce4ui/xfce-dialogs.h>
 #include <libxfce4ui/xfce-gtk-extensions.h>
 #include <libxfce4ui/xfce-gdk-extensions.h>
+#include <libxfce4ui/xfce-spawn.h>
 #include <libxfce4ui/libxfce4ui-private.h>
 #include <libxfce4ui/libxfce4ui-alias.h>
 
-#define NOTNULL(str) ((str) != NULL ? (str) : "")
-
 
 
 static void
@@ -69,12 +68,31 @@ xfce_dialog_show_help_uri (GdkScreen *screen,
                            GtkWindow *parent,
                            GString   *uri)
 {
-  GError *error = NULL;
+  GError   *error = NULL;
+  gchar    *path;
+  gchar    *cmd;
+  gboolean  result;
 
   g_return_if_fail (GDK_IS_SCREEN (screen));
   g_return_if_fail (parent == NULL || GTK_IS_WINDOW (parent));
 
-  if (!gtk_show_uri (screen, uri->str, gtk_get_current_event_time (), &error))
+  path = g_find_program_in_path ("exo-open");
+  if (G_LIKELY (path != NULL))
+    {
+      cmd = g_strdup_printf ("%s --launch WebBrowser '%s'", path, uri->str);
+
+      result = xfce_spawn_command_line_on_screen (screen, cmd, FALSE, TRUE, &error);
+
+      g_free (path);
+      g_free (cmd);
+    }
+  else
+    {
+      /* not very likely to happen, but it is possible exo is not installed */
+      result = gtk_show_uri (screen, uri->str, gtk_get_current_event_time (), &error);
+    }
+
+  if (!result)
     {
       xfce_dialog_show_error (parent, error,
           _("Failed to open web browser for online documentation"));
@@ -124,6 +142,8 @@ xfce_dialog_show_help_response (GtkWidget *dialog,
  * Appart from the @component, @page and @offset the following information
  * is also send to the server: user language and the xfce_version_string().
  *
+ * See also: xfce_dialog_show_help_with_version().
+ *
  * Since 4.10
  */
 void
@@ -131,6 +151,39 @@ xfce_dialog_show_help (GtkWindow   *parent,
                        const gchar *component,
                        const gchar *page,
                        const gchar *offset)
+{
+  xfce_dialog_show_help_with_version (parent, component, page, offset, NULL);
+}
+
+
+
+/**
+ * xfce_dialog_show_help_with_version:
+ * @parent    : transient parent of the dialog, or %NULL.
+ * @component : name of the component opening the help page or %NULL. If the
+ *              value is %NULL the target will be the main page of the
+ *              documentation website.
+ * @page      : subpage of the @component on the website or %NULL.
+ * @offset    : anchor offset in @page or %NULL.
+ * @version   : alternative version, or %NULL to use xfce_version_string().
+ *
+ * Asks the user to visit the online documentation. If confirmed, it will open
+ * the webbrowser and redirect the user to the correct location.
+ *
+ * Appart from the @component, @page and @offset the following information
+ * is also send to the server: user language and the xfce_version_string()
+ * or @version if set.
+ *
+ * See also: xfce_dialog_show_help().
+ *
+ * Since 4.12
+ */
+void
+xfce_dialog_show_help_with_version (GtkWindow   *parent,
+                                    const gchar *component,
+                                    const gchar *page,
+                                    const gchar *offset,
+                                    const gchar *version)
 {
   GtkWidget   *dialog;
   const gchar *name;
@@ -152,9 +205,13 @@ xfce_dialog_show_help (GtkWindow   *parent,
   else
     locale = g_strdup ("C");
 
+  /* use desktop version if none is set */
+  if (version == NULL)
+    version = xfce_version_string ();
+
   /* build the redirect uri */
   uri = g_string_new (MANUAL_WEBSITE);
-  g_string_append_printf (uri, "?version=%s&locale=%s", xfce_version_string (), locale);
+  g_string_append_printf (uri, "?version=%s&locale=%s", version, locale);
   g_free (locale);
 
   if (component != NULL)
@@ -198,13 +255,27 @@ xfce_dialog_show_help (GtkWindow   *parent,
 
   dialog = xfce_message_dialog_new (parent,
                                     _("Online Documentation"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
                                     GTK_STOCK_DIALOG_QUESTION,
+#else
+                                    "dialog-question",
+#endif
                                     primary,
                                     _("You will be redirected to the documentation website "
                                       "where the help pages are maintained and translated."),
-                                    GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                                    GTK_STOCK_CANCEL,
+#else
+                                    "gtk-cancel",
+#endif
+                                    GTK_RESPONSE_NO,
                                     XFCE_BUTTON_TYPE_MIXED,
-                                        GTK_STOCK_HELP, _("_Read Online"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                                        GTK_STOCK_HELP,
+#else
+                                        "help-browser",
+#endif
+                                        _("_Read Online"),
                                         GTK_RESPONSE_YES,
                                     NULL);
   g_free (primary);
@@ -260,9 +331,19 @@ xfce_dialog_show_info (GtkWindow   *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Information"), GTK_STOCK_DIALOG_INFO,
+  xfce_message_dialog (parent, _("Information"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_DIALOG_INFO,
+#else
+                       "dialog-information",
+#endif
                        primary_text, secondary_text,
-                       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_CLOSE,
+#else
+                       "window-close",
+#endif
+                       GTK_RESPONSE_CLOSE, NULL);
 
   g_free (primary_text);
 }
@@ -293,9 +374,19 @@ xfce_dialog_show_warning (GtkWindow   *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Warning"), GTK_STOCK_DIALOG_WARNING,
+  xfce_message_dialog (parent, _("Warning"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_DIALOG_WARNING,
+#else
+                       "dialog-warning",
+#endif
                        primary_text, secondary_text,
-                       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_CLOSE,
+#else
+                       "window-close",
+#endif
+                       GTK_RESPONSE_CLOSE, NULL);
 
   g_free (primary_text);
 }
@@ -327,9 +418,19 @@ xfce_dialog_show_error (GtkWindow    *parent,
   primary_text = g_strdup_vprintf (primary_format, args);
   va_end (args);
 
-  xfce_message_dialog (parent, _("Error"), GTK_STOCK_DIALOG_ERROR,
+  xfce_message_dialog (parent, _("Error"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_DIALOG_ERROR,
+#else
+                       "dialog-error",
+#endif
                        primary_text, error ? error->message : NULL,
-                       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+#if !GTK_CHECK_VERSION (3, 10, 0)
+                       GTK_STOCK_CLOSE,
+#else
+                       "window-close",
+#endif
+                       GTK_RESPONSE_CLOSE, NULL);
 
   g_free (primary_text);
 }
@@ -373,13 +474,29 @@ xfce_dialog_confirm (GtkWindow   *parent,
   va_end (args);
 
   /* whether this will be a yes/no dialog */
-  if (stock_id != NULL && strcmp (stock_id, GTK_STOCK_YES) == 0)
-    no_stock_id = GTK_STOCK_NO;
+  if (stock_id != NULL && (strcmp (stock_id, "gtk-yes") == 0 || strcmp (stock_id, "yes")) == 0)
+    {
+#if GTK_CHECK_VERSION (3, 10, 0)
+      no_stock_id = "gtk-no";
+#else
+      no_stock_id = GTK_STOCK_NO;
+#endif
+    }
   else
-    no_stock_id = GTK_STOCK_CANCEL;
+    {
+#if GTK_CHECK_VERSION (3, 10, 0)
+      no_stock_id = "gtk-cancel";
+#else
+      no_stock_id = GTK_STOCK_CANCEL;
+#endif
+    }
 
   response_id = xfce_message_dialog (parent, _("Question"),
+#if !GTK_CHECK_VERSION (3, 10, 0)
                                      GTK_STOCK_DIALOG_QUESTION,
+#else
+                                     "dialog-question",
+#endif
                                      primary_text, secondary_text,
                                      no_stock_id, GTK_RESPONSE_NO,
                                      XFCE_BUTTON_TYPE_MIXED, stock_id,
@@ -460,8 +577,14 @@ xfce_message_dialog_new_valist (GtkWindow   *parent,
   if (icon_stock_id != NULL)
     {
       /* set dialog and window icon */
+#if GTK_CHECK_VERSION (3, 10, 0)
+      image = gtk_image_new_from_icon_name (icon_stock_id, GTK_ICON_SIZE_DIALOG);
+      gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), image);
+#else
       image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
       gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+#endif
+
       gtk_widget_show (image);
       gtk_window_set_icon_name (GTK_WINDOW (dialog), icon_stock_id);
     }
